from decimal import Decimal

import matplotlib.pyplot as plt
import numpy as np

from mpl_toolkits.mplot3d import Axes3D
from matplotlib.ticker import PercentFormatter

import matplotlib.patches as mpatches

import matplotlib.animation as animation

import time
        
# global variables
content_array = []
l = m = n = x = y = z = rad = 0.0

# Getting inputs from input file saved on content_array
def file_read():
    with open("InputFile.txt") as f:
        for line in f:
            if not line.startswith("#") and not line == "\n":
                content_array.append( (line.rstrip()).lstrip())
    
    if (len(content_array) < 6 or len(content_array) > 6):
        print ("Error: Please provide 6 inputs")
        return False    
    else:
        print ("size = " + str(len(content_array)))
        print (content_array)
        return True    

def assign_variables():
    print (content_array)
    print()

    # assigning values from content_array indexes    
    try:
        global l
        global m
        global n
        global x
        global y
        global z
        global rad
        m = float(content_array[0])
        n = float(content_array[1])
        x = (float(content_array[2]))
        y = (float(content_array[3]))
        z = (float(content_array[4]))
        rad = float(content_array[5])
        
        l = np.absolute(np.sqrt(1-((m*m)+(n*n))))
    except ValueError:
        print ("Please enter numeric inputs")
        exit()

# Print desired output in Output File, expressing to
# 4 s.f.
def file_write (TMin, TMax):
    fname = "OutputFile.txt"
    f = open(fname, 'w')
    f.write("For user input: " )
    f.write('\n')    
    f.write("l: " + str('%.3E' % Decimal(l)) + '\n')
    f.write("m: " + str('%.3E' % Decimal(m)) + '\n')
    f.write("n: " + str('%.3E' % Decimal(n)) + '\n')
    f.write("x: " + str('%.3E' % Decimal(x)) + '\n')
    f.write("y: " + str('%.3E' % Decimal(y)) + '\n')
    f.write("z: " + str('%.3E' % Decimal(z)) + '\n')
    f.write("radius: " + str('%.3E' % Decimal(rad)) + '\n')
    
    f.write('\n')
    f.write('\n')
    f.write("The Minimum Time Delay is: " + str('%.3E' % Decimal(TMin)))
    f.write('\t')
    f.write("The Maximum Time Delay is: " + str('%.3E' % Decimal(TMax)))
    f.close

# Concatenates all S for all 5 time delay conditions to allow for
# easy calculation of T
def concatenateS(S0, S0to90, S90, S90to180, S180, S180to270, S270, SQ270to360):
    print("Concatenating now")
    sList = []
    S = np.array(sList)
    if (S0.size != 0):
        S = np.concatenate([S, S0])
    if (S0to90.size != 0):
        S = np.concatenate([S, S0to90])
    if (S90.size != 0):
        S = np.concatenate([S, S90])
    if (S90to180.size != 0):
        S = np.concatenate([S, S90to180])
    if (S180.size != 0):
        S = np.concatenate([S, S180])
    if (S180to270.size != 0):
        S = np.concatenate([S, S180to270])
    if (S270.size != 0):
        S = np.concatenate([S, S270])
    if (S270to360.size != 0):
        S = np.concatenate([S, S270to360])
    print ("Final S = " + str(S))
    return(S)

ret = file_read()
if(ret):
    assign_variables()
    
else:
    print ("Error: Input file could not be read")
    exit()

print ("l = " + str(l))
print ("m = " + str(m))
print ("n = " + str(n))
print ("x = " + str(x))
print ("y = " + str(y))
print ("z = " + str(z))
print ("rad = " + str(rad))

xyzDist = np.sqrt(x*x + y*y + z*z)
print ("xyzDist = " + str(xyzDist))


radSqrd = rad*rad 

fig = plt.figure()
axis = fig.gca()

# getting the coordinates of a point on the disk
xCir = np.random.uniform(low=-rad, high=rad, size=100000000)

yCir = np.random.uniform(low=-np.sqrt(radSqrd-(xCir*xCir)),
                                 
                                 high=np.sqrt(radSqrd-(xCir*xCir)), size=100000000)

xCir = xCir + x
yCir = yCir + y
zCir = z

# calculating the distance to the point on the disk
SPx = xCir
SPy = yCir
SPz = zCir
print ("SPx = " + str(SPx))
print ("SPy = " + str(SPy))
print ("SPz = " + str(SPz))

SP = np.sqrt( (SPx * SPx) + (SPy * SPy) + (SPz * SPz) )
print ("SP = " + str(SP) )

print ("SPx = " + str(SPx) )



print(z/xyzDist)
print(n)
print(y/xyzDist)

print(m)

# calculating S for all 5 light delay conditions
Phi = np.arccos((SPx*l + SPy*m + SPz*n)/((np.sqrt(SPx*SPx + SPy*SPy + SPz*SPz)*np.sqrt(l*l + m*m + n*n))))
print("Phi = "+str(Phi))

Phi0 = Phi[Phi == 0]
print("Phi0 = "+str(Phi0))

Phi0to90 = Phi[abs(Phi) > 0]
Phi0to90 = Phi0to90[abs(Phi) < np.pi/2]
print("Phi0to90 = "+str(Phi0to90))

Phi90 = Phi[Phi == np.pi/2]
print("Phi90 = "+str(Phi90))

Phi90to180 = Phi[abs(Phi) > np.pi/2]
Phi90to180 = Phi90to180[abs(Phi90to180) < np.pi]
print("Phi90to180 = "+str(Phi90to180))

Phi180 = Phi[Phi == np.pi]
print("Phi180 = "+str(Phi180))



Phi180to270 =  Phi[abs(Phi) > np.pi]
Phi180to270 =  Phi180to270[abs(Phi180to270) < (3*np.pi)/2]
#print("Phi180to270 = "+str(Phi180to270))

Phi270 = Phi[Phi == (3*np.pi)/2]
#print("Phi270 = "+str(Phi270))

Phi270to360 = Phi[abs(Phi) > (3*np.pi)/2]
Phi270to360 = Phi270to360[abs(Phi270to360) < (2*np.pi)]                  
#print("Phi270to360 = "+str(Phi270to360))

S0 = S0to90 = S90 = S90to180 = S180 = S180to270 = S270 = S270to360 = Phi[Phi != Phi]


if (Phi0.size != 0):
    S0 = Phi0

if (Phi0to90.size != 0):
    SQ0to90 = np.cos(Phi0to90)*SP
    S0to90 = SP - SQ0to90

if (Phi90.size != 0):
    S90 = SP[Phi == np.pi/2]
    #S90 = SP90

if (Phi90to180.size != 0):
    PQ90to180 = np.cos(np.pi - Phi90to180)*SP
    S90to180 = SP + PQ90to180

if (Phi180.size != 0):
    S180 = SP[Phi == np.pi] * 2
    #S180 = 2*SP180

PQ180to270 = np.cos(np.pi - Phi)*SP
S180to270 = SP + PQ180to270

S270 = SP[Phi == (3*np.pi)/2]
#S90 = SP90

SQ270to360 = np.cos(Phi)*SP
S270to360 = SP - SQ270to360



# Calling method concatenateS and perform calculation of T on
# concatenated array
S = concatenateS(S0, S0to90, S90, S90to180, S180, S180to270, S270, SQ270to360)

T = S/299792458
print ("T = "+str(T))
 

TMin = np.amin(T)
print(TMin)

TMax = np.amax (T)
print(TMax)

# Creating the bins for display
d = (TMax - TMin)/(41 - 1)

TBin = [None] * 41

for count in range (41):
        TBin[count] = TMin + ((count + 1) - 1) * d
           

bins = TBin

# Adding on-graph text        
axis.text(0.8, 0.9,'l = '+ str('%.2E' % Decimal(l)), horizontalalignment='center', verticalalignment='center', transform = axis.transAxes)
axis.text(0.8, 0.8,'m = '+ str('%.2E' % Decimal(m)), horizontalalignment='center', verticalalignment='center', transform = axis.transAxes)
axis.text(0.8, 0.7,'n = '+ str('%.2E' % Decimal(n)), horizontalalignment='center', verticalalignment='center', transform = axis.transAxes)
axis.text(0.8, 0.6,'x = '+ str('%.2E' % Decimal(x)), horizontalalignment='center', verticalalignment='center', transform = axis.transAxes)
axis.text(0.8, 0.5,'y = '+ str('%.2E' % Decimal(y)), horizontalalignment='center', verticalalignment='center', transform = axis.transAxes)
axis.text(0.8, 0.4,'z = '+ str('%.2E' % Decimal(z)), horizontalalignment='center', verticalalignment='center', transform = axis.transAxes)
axis.text(0.8, 0.3,'r = '+ str('%.2E' % Decimal(rad)), horizontalalignment='center', verticalalignment='center', transform = axis.transAxes)

# plotting the data
plt.hist(T, bins, weights=np.ones(len(T))/len(T)*(np.pi*radSqrd)/(4*np.pi*(xyzDist*xyzDist)), edgecolor='black')
plt.xlabel('Light Delay/s')
plt.ylabel('Probability of a Light Delay Falling within a Light Delay Bin')

# updating the output file
file_write(TMin, TMax)
print ("plotting")
print (" ");
plt.show()


